import { Directive, ElementRef, Renderer2, Input, NgModule } from '@angular/core';
import { Location, LocationStrategy, PathLocationStrategy } from '@angular/common';
import { isUndefined, isString } from 'util';
import { NavigationStart, NavigationEnd, Router } from '@angular/router';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxRouteHighlighterConfig {
    /**
     * @param {?=} config
     */
    constructor(config) {
        this.config = config;
    }
    /**
     * @param {?} url
     * @param {?=} className
     * @return {?}
     */
    getNormalConfig(url, className) {
        this.config = {
            className: className ? className : 'active',
            links: url,
            strict: false
        };
        return new NgxRouteHighlighterConfig(this.config);
    }
    /**
     * @param {?} url
     * @param {?=} className
     * @return {?}
     */
    getStrictConfig(url, className) {
        this.config = {
            className: className ? className : 'active',
            links: url,
            strict: true
        };
        return new NgxRouteHighlighterConfig(this.config);
    }
}
if (false) {
    /** @type {?} */
    NgxRouteHighlighterConfig.prototype.config;
}
/**
 * @record
 */
function LinksEntity() { }
if (false) {
    /** @type {?} */
    LinksEntity.prototype.links;
    /** @type {?} */
    LinksEntity.prototype.className;
    /** @type {?|undefined} */
    LinksEntity.prototype.strict;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class RouteHighlighterDirective {
    /**
     * @param {?} el
     * @param {?} location
     * @param {?} r2
     * @param {?} router
     */
    constructor(el, location, r2, router) {
        this.el = el;
        this.location = location;
        this.r2 = r2;
        this.router = router;
        this.defaultConfig = {
            className: 'active',
            links: null,
            strict: false
        };
        this.ngxRouteHighlighter = new NgxRouteHighlighterConfig(this.defaultConfig);
        this.activeClass = 'active';
        // console.clear();
        router.events.subscribe((/**
         * @param {?} event
         * @return {?}
         */
        event => {
            if (event instanceof NavigationStart) {
                this.r2.removeClass(this.el.nativeElement, this.activeClass);
            }
            else if (event instanceof NavigationEnd) {
                this.setClass();
            }
        }));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.ngxRouteHighlighter = new NgxRouteHighlighterConfig(this.defaultConfig);
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (!isUndefined(changes.ngxRouteHighlighter.currentValue)) {
            this.ngxRouteHighlighter = changes.ngxRouteHighlighter.currentValue;
        }
        if (this.ngxRouteHighlighter.config) {
            this.activeClass = this.ngxRouteHighlighter.config.className ? this.ngxRouteHighlighter.config.className : 'active';
        }
        else {
            this.activeClass = 'active';
        }
    }
    /**
     * @return {?}
     */
    setClass() {
        if (this.linkMatcher(this.ngxRouteHighlighter.config.links, this.ngxRouteHighlighter.config.strict)) {
            this.r2.addClass(this.el.nativeElement, this.activeClass);
        }
    }
    /**
     * @param {?} links
     * @param {?=} strict
     * @return {?}
     */
    linkMatcher(links, strict) {
        /** @type {?} */
        const foundMatch = [];
        if (!strict) {
            strict = false;
        }
        else {
            console.warn('strict is dapricated it will not have any effect as of now and it will be removed after version 0.1.0');
        }
        if (isString(links)) {
            links = [links];
        }
        if (links) {
            links.forEach((/**
             * @param {?} link
             * @param {?} index
             * @return {?}
             */
            (link, index) => {
                /** @type {?} */
                const $r = this.regExpGen(link);
                foundMatch.push($r.test(this.location.path()));
            }));
        }
        // console.log(foundMatch);
        return foundMatch.indexOf(true) !== -1;
    }
    /**
     * @param {?} link
     * @return {?}
     */
    regExpGen(link) {
        /** @type {?} */
        const ragexBaseString = link.replace(new RegExp(/(\/\*)/, 'gmi'), '\/[a-z0-9]+');
        return new RegExp(String.raw `(${ragexBaseString})`, 'gmi');
    }
}
RouteHighlighterDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxRouteHighlighter]',
                providers: [Location, { provide: LocationStrategy, useClass: PathLocationStrategy }],
            },] }
];
/** @nocollapse */
RouteHighlighterDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Location },
    { type: Renderer2 },
    { type: Router }
];
RouteHighlighterDirective.propDecorators = {
    ngxRouteHighlighter: [{ type: Input }],
    activeClass: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    RouteHighlighterDirective.prototype.defaultConfig;
    /** @type {?} */
    RouteHighlighterDirective.prototype.ngxRouteHighlighter;
    /** @type {?} */
    RouteHighlighterDirective.prototype.activeClass;
    /**
     * @type {?}
     * @private
     */
    RouteHighlighterDirective.prototype.el;
    /**
     * @type {?}
     * @private
     */
    RouteHighlighterDirective.prototype.location;
    /**
     * @type {?}
     * @private
     */
    RouteHighlighterDirective.prototype.r2;
    /**
     * @type {?}
     * @private
     */
    RouteHighlighterDirective.prototype.router;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxRouteHighlighterModule {
    /**
     * @param {?} router
     * @return {?}
     */
    static forRoot(router) {
        return {
            ngModule: NgxRouteHighlighterModule,
            imports: [
                router,
            ]
        };
    }
}
NgxRouteHighlighterModule.decorators = [
    { type: NgModule, args: [{
                declarations: [RouteHighlighterDirective],
                imports: [],
                exports: [RouteHighlighterDirective],
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxRouteHighlighterConfig, NgxRouteHighlighterModule, RouteHighlighterDirective };
//# sourceMappingURL=ngx-route-highlighter.js.map
